-- Create stub
ScalzBam = LibStub("AceAddon-3.0"):NewAddon("ScalzBam", "AceConsole-3.0", "AceEvent-3.0")


-----------------------------------------------------

function ScalzBam:Print(msg)
	print("|cfff26113ScalzBam|r:", msg)
end

function ScalzBam:Error(msg)
	print("|cffff0000Error|r:", msg)
end

function ScalzBam:Post(spell, dmg, inInstance, isShowoff)
	if (self.db.char.channel) then
		-- handle blocked say/yell
		if not (inInstance) and ((self.db.char.channel == "say") or (self.db.char.channel == "yell")) then
			self:Error("Could not display in '/" .. self.db.char.channel .. "' channel")
			self:Error("You are not in an instance")
		else
			if (isShowoff) then
				SendChatMessage("[ScalzBam] " .. spell .. " has crit for " .. dmg .. "!", string.upper(self.db.char.channel))
			else
				SendChatMessage("[ScalzBam] BAM! " .. spell .. " crit for " .. dmg .. "!", string.upper(self.db.char.channel))
			end
		end
	else

	end
end

function ScalzBam:HandleCombatEvent(spellName, dmg, inInstance)
	local currentRecord = self.db.char.records[spellName]

	-- First time
	if (currentRecord == nil) then
		currentRecord = 0
	end

	if (dmg > currentRecord) then
		-- new record
		self.db.char.records[spellName] = dmg

		-- play audio if enabled
		if (self.db.char.audio) then
			PlaySoundFile("Interface\\AddOns\\ScalzBam\\crazy.ogg")
		end

		-- print
		self:Print("New highscore, " .. spellName .. " with " .. dmg .. "!")

		-- tell
		self:Post(spellName, dmg, inInstance)
	end
end

function ScalzBam:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ScalzBamDB")
	if (self.db.char.records == nil) then
		self.db.char.records = {}
	end
	-- Clear old AA key
	self.db.char.records["AA"] = nil

	if (self.db.char.enabled) then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end

	-- TESTING
	--self.db.char.records["Shoot"] = nil
end

function ScalzBam:OnEnable()
end

function ScalzBam:OnDisable()
end

function ScalzBam:SetEnabled(info, val)
	self.db.char.enabled = val
	
	if (val) then
		self:Print("Enabled addon")
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else
		self:Print("Disabled addon")
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function ScalzBam:GetEnabled(info)
	return self.db.char.enabled
end

function ScalzBam:SetAudio(info, val)
	self.db.char.audio = val
	
	if (val) then
		self:Print("Enabled audio")
	else
		self:Print("Disabled audio")
	end
end

function ScalzBam:GetAudio(info)
	return self.db.char.audio
end

function ScalzBam:SetChannel(info, val)
	if (val == "none") then
		self.db.char.channel = nil
		self:Print("Channel cleared")
		return
	end

	self.db.char.channel = val
	self:Print("Channel set to '/" .. val .. "'")

	if (val == "say") or (val == "yell") then
		self:Print("NOTE: This channel only works in instances!")
	end
end

function ScalzBam:GetChannel(info)
	return self.db.char.channel
end

function ScalzBam:SetDebug(info, val)
	self.db.char.debug = val

	if (val) then
		self:Print("Debug mode ON")
	else
		self:Print("Debug mode OFF")
	end
end

function ScalzBam:GetDebug()
	return self.db.char.debug
end

function ScalzBam:NEW_ShowHighscores()
	local frame = ScalzBam:GetModule("ScalzBam_HighscoreFrame")

	local count = 0
	for spell, damage in pairs(self.db.char.records) do
		frame:AddRow(spell, damage, count+1)
		count = count + 1
	end
	if (count == 0) then
		frame:Empty()
	end

	frame:Show()
end

function ScalzBam:ShowHighscores()
	local AceGUI = LibStub("AceGUI-3.0")

	-- Create a container frame
	local frame = AceGUI:Create("Frame")
	frame:SetWidth(380)
	frame:SetHeight(400)
	frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	frame:SetTitle("Highscores")
	frame:SetStatusText("Are you proud of yourself?")
	frame:SetLayout("Fill")

	-- Create scrollable container
	local scroll = AceGUI:Create("ScrollFrame")
	scroll:SetLayout("List")
	frame:AddChild(scroll)
	
	--
	-- Iterate all highscores
	--
	local count = 0
	for spell, damage in pairs(self.db.char.records) do
		-- Create highscore container
		local container = AceGUI:Create("SimpleGroup")
		container:SetAutoAdjustHeight(false)
		container:SetLayout("Flow")
		container:SetHeight(100)

		local _, __, spellIcon = GetSpellInfo(spell)
		-- auto attack fix
		if (spell == "Auto attack") then
			spellIcon = "Interface\\ICONS\\Ability_MeleeDamage"
		end
		local i = AceGUI:Create("Icon")
		i:SetImage(spellIcon)
		i:SetImageSize(48, 48)
		i:SetCallback("OnClick", function(button)
			if (self.db.char.channel) then
				local _, type = GetInstanceInfo()
				if (type == "none") then isInInstance = false
				else isInInstance = true end
				self:Post(spell, damage, isInInstance, true)
			else
				self:Print("You do not have a channel selected")
				self:Print("Select one with '/bam channel'")
			end
		end)
		container:AddChild(i)

		local l = AceGUI:Create("Label")
		l:SetFullWidth(true)
		l:SetFont("Fonts\\FRIZQT__.TTF", 16)
		l:SetText("|cffe6d417" .. spell .. "|r" .. " with |cffff0000" .. damage .. "|r damage!")
		container:AddChild(l)

		count = count + 1

		scroll:AddChild(container)
	end
	if (count == 0) then
		local l = AceGUI:Create("Label")
		l:SetText("No highscores recorded yet!")
		container:AddChild(l)
	end

	frame:DoLayout()
end

function ScalzBam:ClearHighscores()
	self.db.char.records = {}
	self:Print("Cleared highscores")
end

function ScalzBam:COMBAT_LOG_EVENT_UNFILTERED()
	local eventInfo = { CombatLogGetCurrentEventInfo() }
	local type = eventInfo[2]
	local sourceGUID = eventInfo[4]
	local sourceFlags = eventInfo[6]
	local isTotem = (bit.band(sourceFlags, 0x00002001) == 0x00002001)

	if (self.db.char.debug) then
		print("Type: " .. type)
		print("SourceFlags: " .. string.format( "0x%8.8X", sourceFlags ))
	end

	-- Ignore from anything other than self, pet or totem
	if not (sourceGUID == UnitGUID("player")) and not (sourceGUID == UnitGUID("pet")) and not (isTotem) then
		return
	end

	local _, instanceType = GetInstanceInfo()
	if (instanceType == "none") then instanceType = false
	else instanceType = true
	end

	local spellName, amount, isCrit

	if (type == "SWING_DAMAGE") then
		isCrit = eventInfo[18]
		spellName = "Auto attack"
		amount = eventInfo[12]

	elseif (type == "SPELL_DAMAGE") or (type == "RANGE_DAMAGE") then
		isCrit = eventInfo[21]
		spellName = eventInfo[13]
		amount = eventInfo[15]

	elseif (type == "SPELL_HEAL") then
		isCrit = eventInfo[18]
		spellName = eventInfo[13]
		amount = eventInfo[15]
	else
		-- Ignore other types
		return;
	end

	-- Ignore non crits
	if not (isCrit) then
		return
	end

	self:HandleCombatEvent(spellName, amount, instanceType)
end


-------------------------------------------------------------------------


-- Register events
ScalzBam:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

-- Register options
local options = {
    name = "ScalzBam",
    handler = ScalzBam,
    type = 'group',
    args = {
    	enable = {
			name = "Addon enable",
			desc = "   Enables / disables the addon",
			type = "toggle",
			set = "SetEnabled",
			get = "GetEnabled"
		},
		channel = {
			name = "Set channel",
			desc = " Set text channel for highscore output",
			type = "select",
			values = {
				say="        /s",
				yell="       /y",
				party="      /p",
				raid="       /r",
				rw="          /rw",
				guild="     /g",
				officer="   /o",
				none="      clear channel",
			},
			get = "GetChannel",
			set = "SetChannel"
		},
        audio = {
        	name = 'Audio enable',
        	desc = '     Enables / disables audio on new highscore',
            type = 'toggle',
            set = 'SetAudio',
            get = 'GetAudio',
        },
        hs = {
        	name = 'Audio enable',
        	desc = '          Show all current highscores',
            type = 'execute',
            func = "ShowHighscores"
        },
        clear = {
        	name = 'Clear highscores',
        	desc = '      Clear entire collection of highscores',
            type = 'execute',
            func = "ClearHighscores",
        },
        debug = {
        	name = "Debug mode",
        	desc = "    Turn on/off debug mode",
        	type = "toggle",
        	set  = "SetDebug",
        	get  = "GetDebug"
        }
    },
}

LibStub("AceConfig-3.0"):RegisterOptionsTable("ScalzBam", options, {"bam", "scalzbam"})